
console.log('Redwax SignText: background started');

var csPorts = [];
var nativePort;

function nativeMessaged(m) {
    /*
     * Native message response is handled here. Send the message
     * to the correctly numbered content script.
     */
    window.csPorts.forEach(p => {
        if (p.sender.tab.id == m.id) {
            p.postMessage({ uuid: m.uuid, response: m.response, contentType: m.contentType });
        }
    });
}

function nativeDisconnected(p) {
    window.nativePort.disconnect();
    window.nativePort = undefined;
    console.log('Redwax SignText: native port disconnected.');
}

function connected(p) {
    window.csPorts[p.sender.tab.id] = p;
    p.onMessage.addListener(function(m) {
        
        try {
            /*
             * Let's try send the message on the assumption the port is
             * open and works. On failure, try to open the port and retry.
             */
            window.nativePort.postMessage({ id: p.sender.tab.id, uuid: m.uuid, request: m.request, contentType: m.contentType });
        }
        catch (exception) {
            
            /*
             * Not able to send the message. (Re)open the port and try
             * once more.
             */
            window.nativePort = browser.runtime.connectNative('Redwax-SignText');
            window.nativePort.onMessage.addListener(nativeMessaged);
            window.nativePort.onDisconnect.addListener(nativeDisconnected);
            
            /*
             * One more try to send the message.
             */
            try {
                window.nativePort.postMessage({ id: p.sender.tab.id, uuid: m.uuid, request: m.request, contentType: m.contentType });
            }
            catch (exception) {
                
                /*
                 * No luck, send a message back to the content script with
                 * the bad news.
                 */
                p.postMessage({ uuid: m.uuid, error: 'error:postNativeMessageFailed', exception: exception });
                
            }
            
        }
        
    });
    p.onDisconnect.addListener(function(p) {
        console.log('Redwax SignText: tab ' + p.sender.tab.id + ' disconnected.');
        delete csPorts[p.sender.tab.id];
    });
}

browser.runtime.onConnect.addListener(connected);

console.log('Redwax SignText: background ended');
