//
//  SafariWebExtensionHandler.m
//  Redwax SignText Extension
//
//  Created by Graham Leggett on 2021/08/10.
//

#import "SafariWebExtensionHandler.h"

#import <SafariServices/SafariServices.h>

#import <Cocoa/Cocoa.h>

#if __MAC_OS_X_VERSION_MIN_REQUIRED < 110000
NSString * const SFExtensionMessageKey = @"message";
#endif

@implementation SafariWebExtensionHandler

- (void)beginRequestWithExtensionContext:(NSExtensionContext *)context {

    NSExtensionItem *response = [[NSExtensionItem alloc] init];

    id message = [context.inputItems.firstObject userInfo][SFExtensionMessageKey];
    NSLog(@"Received message from browser.runtime.sendNativeMessage: %@", message);

    /*
     * Message ID? If not, error.
     */
    id messageId = message[@"id"];
    if (messageId == NULL) {
        NSLog(@"Redwax SignText Extension: message ID is missing, skipping.");
        response.userInfo = @{ SFExtensionMessageKey: @{ @"error": @"error:missingId" } };
        [context completeRequestReturningItems:@[ response ] completionHandler:nil];
        return;
    }

    /*
     * Message UUID? If not, error.
     */
    id messageUuid = message[@"uuid"];
    if (messageUuid == NULL) {
        NSLog(@"Redwax SignText Extension: message UUID is missing, skipping.");
        response.userInfo = @{ SFExtensionMessageKey: @{ @"id": messageId, @"error": @"error:missingUuid" } };
        [context completeRequestReturningItems:@[ response ] completionHandler:nil];
        return;
    }


    id messageRequest = message[@"request"];

    /*
     * Incoming data to sign.
     */
    if ([messageRequest isKindOfClass:[NSString class]]) {
        /*
         * Message Content-Type? If not, error.
         */
        id messageContentType = message[@"contentType"];
        if (messageContentType == NULL) {
            NSLog(@"Redwax SignText Extension: message Content-Type is missing, skipping.");
            response.userInfo = @{ SFExtensionMessageKey: @{ @"id": messageId, @"uuid": messageUuid, @"error": @"error:missingContentType" } };
            [context completeRequestReturningItems:@[ response ] completionHandler:nil];
            return;
        }
        else {
            response.userInfo = @{ SFExtensionMessageKey: @{ @"id": messageId, @"uuid": messageUuid, @"response": @YES } };
            [context completeRequestReturningItems:@[ response ] completionHandler:nil];
            return;
        }

    }

    /*
     * No more incoming data, do the sign.
     */
    else if ([messageRequest isKindOfClass:[NSDictionary class]]) {
        // do the sign, return the first chunk

        response.userInfo = @{ SFExtensionMessageKey: @{ @"id": messageId, @"uuid": messageUuid, @"contentType": @"application/pkcs7-mime", @"response": @"Signed text" } };
        [context completeRequestReturningItems:@[ response ] completionHandler:nil];
        return;

    }

    /*
     * No more outgoing data, we're done.
     */
    else if ([messageRequest isKindOfClass:[NSNumber class]]) {
        // no more chunks to return, we're done

        response.userInfo = @{ SFExtensionMessageKey: @{ @"id": messageId, @"uuid": messageUuid, @"response": @NO } };
        [context completeRequestReturningItems:@[ response ] completionHandler:nil];
        return;
    }

    /*
     * Not a clue, sorry.
     */
    else {
        response.userInfo = @{ SFExtensionMessageKey: @{ @"id": messageId, @"uuid": messageUuid, @"error": @"error:requestUnrecognised" } };
        [context completeRequestReturningItems:@[ response ] completionHandler:nil];
        return;
    }


//    NSAlert* alert = [[NSAlert alloc]init];
//    [alert setMessageText:message[@"request"]];
//    [alert setInformativeText:@"Hello"];
//    [alert runModal];

}

@end
