browser.runtime.sendMessage({ greeting: "hello" }).then((response) => {
    console.log("Received response: ", response);
});

browser.runtime.onMessage.addListener((request, sender, sendResponse) => {
    console.log("Received request: ", request);
});

window.addEventListener("message", (event) => {
  // Do we trust the sender of this message?
//  if (event.origin !== "http://example.com:8080")
//    return;

  // event.source is window.opener
  // event.data is "hello there!"

  // Assuming you've verified the origin of the received message (which
  // you must do in any case), a convenient idiom for replying to a
  // message is to call postMessage on event.source and provide
  // event.origin as the targetOrigin.
  event.source.postMessage("hi there " + event.origin + "!  the secret response " +
                           "is: rheeeeet!",
                           event.origin);
}, false);


class SignTextResponse {

    constructor(context, stream) {
        this.context = context;
        this.stream = stream;
    }

    start(controller) {
        console.log("signText: start");

        this.port = browser.runtime.connect({name:"signText"});

        let message = {
            message: "signText",
            payload: context
        };

        this.port.postMessage(message);

    }

    pull(controller) {
        console.log("signText: pull");
    }

    cancel(reason) {
        console.log("signText: cancel");
    }

}

function signText(text, options, ...CAs) {

    /* legacy - pass a string, we return a string */
    if (typeof text === 'string') {
        var requestStream = new ReadableStream({
            read() {
                this.push(text);
                this.push(null);
            }
        });
    }
    /* consume a ReadableStream */
    else if (text instanceof ReadableStream) {
        var requestStream = text;
    }
    else {
        // error - parameter not recognised
    }

    let context = {
        message: "signText",
        options: options,
        CAs: CAs,
        protocol: location.protocol,
        hostname: location.hostname
    };

    const responseStream = new ReadableStream(
                    new SignTextResponse(context, requestStream));

    /* legacy - pass a string, we return a string */
    if (typeof text === 'string') {
        return new Response(responseStream).text();
    }
    /* return a ReadableStream */
    else {
        return responseStream;
    }

}

exportFunction(signText, window.crypto, {defineAs: "signText"});
